@@ -45,8 +45,6 @@ namespace Tiled {
  */
 class TILEDSHARED_EXPORT ImageLayer : public Layer
 {
-    Q_OBJECT
-
 public:
     ImageLayer(const QString &name, int x, int y);
     ~ImageLayer() override;
@@ -106,9 +104,10 @@ public:
      * @return <code>true</code> if loading was successful, otherwise
      *         returns <code>false</code>
      */
-    bool loadFromImage(const QImage &image, const QUrl &source);
+    bool loadFromImage(const QPixmap &image, const QUrl &source);
     bool loadFromImage(const QImage &image, const QString &source);
     bool loadFromImage(const QUrl &url);
+    bool loadFromImage(const ImageReference &image);
 
     /**
      * Returns true if no image source has been set.
