@@ -33,6 +33,7 @@
 #include "tile.h"
 #include "tilelayer.h"
 #include "tileset.h"
+#include "objectgroup.h"
 
 #include <QtCore/qmath.h>
 
@@ -74,30 +75,27 @@ QRect OrthogonalRenderer::boundingRect(const QRect &rect) const
 
 QRectF OrthogonalRenderer::boundingRect(const MapObject *object) const
 {
-    const QRectF bounds = object->bounds();
+    QRectF bounds = object->bounds();
+    bounds.translate(-alignmentOffset(bounds, object->alignment(map())));
 
     QRectF boundingRect;
 
     if (!object->cell().isEmpty()) {
-        const QSizeF objectSize { object->size() };
-
-        QSizeF scale { 1.0, 1.0 };
-        QPoint tileOffset;
-
         if (const Tile *tile = object->cell().tile()) {
-            QSize imgSize = tile->size();
-            if (!imgSize.isNull()) {
-                scale = QSizeF(objectSize.width() / imgSize.width(),
-                               objectSize.height() / imgSize.height());
+            QPointF tileOffset = tile->offset();
+            const QSize tileSize = tile->size();
+            if (!tileSize.isNull()) {
+                const QSizeF scale {
+                    bounds.width() / tileSize.width(),
+                    bounds.height() / tileSize.height()
+                };
+                tileOffset.rx() *= scale.width();
+                tileOffset.ry() *= scale.height();
             }
-            tileOffset = tile->offset();
+            bounds.translate(tileOffset);
         }
 
-        const QPointF bottomLeft = bounds.topLeft();
-        boundingRect = QRectF(bottomLeft.x() + (tileOffset.x() * scale.width()),
-                              bottomLeft.y() + (tileOffset.y() * scale.height()) - objectSize.height(),
-                              objectSize.width(),
-                              objectSize.height()).adjusted(-1, -1, 1, 1);
+        boundingRect = bounds.adjusted(-1, -1, 1, 1);
     } else {
         qreal extraSpace = qMax(objectLineWidth(), qreal(1));
 
@@ -110,10 +108,10 @@ QRectF OrthogonalRenderer::boundingRect(const MapObject *object) const
                                                10 + extraSpace + 1,
                                                10 + extraSpace + 1);
             } else {
-            boundingRect = bounds.adjusted(-extraSpace,
-                                           -extraSpace,
-                                           extraSpace + 1,
-                                           extraSpace + 1);
+                boundingRect = bounds.adjusted(-extraSpace,
+                                               -extraSpace,
+                                               extraSpace + 1,
+                                               extraSpace + 1);
             }
             break;
 
@@ -141,7 +139,7 @@ QRectF OrthogonalRenderer::boundingRect(const MapObject *object) const
         }
 
         case MapObject::Text:
-            boundingRect = object->bounds();
+            boundingRect = bounds;
             break;
         }
     }
@@ -153,10 +151,11 @@ QPainterPath OrthogonalRenderer::shape(const MapObject *object) const
 {
     QPainterPath path;
 
+    QRectF bounds = object->bounds();
+    bounds.translate(-alignmentOffset(bounds, object->alignment(map())));
+
     switch (object->shape()) {
     case MapObject::Rectangle: {
-        const QRectF bounds = object->bounds();
-
         if (bounds.isNull())
             path.addRect(object->x() - 10, object->y() - 10, 20, 20);
         else
@@ -176,8 +175,6 @@ QPainterPath OrthogonalRenderer::shape(const MapObject *object) const
         break;
     }
     case MapObject::Ellipse: {
-        const QRectF bounds = object->bounds();
-
         if (bounds.isNull())
             path.addEllipse(bounds.topLeft(), 20, 20);
         else
@@ -188,7 +185,7 @@ QPainterPath OrthogonalRenderer::shape(const MapObject *object) const
         path = pointShape(object->position());
         break;
     case MapObject::Text: {
-        path.addRect(object->bounds());
+        path.addRect(bounds);
         break;
     }
     }
@@ -202,10 +199,7 @@ QPainterPath OrthogonalRenderer::interactionShape(const MapObject *object) const
 
     switch (object->shape()) {
     case MapObject::Rectangle:
-        if (object->isTileObject())
-            path.addRect(boundingRect(object));
-        else
-            path = shape(object);
+        path.addRect(boundingRect(object));
         break;
     case MapObject::Polyline: {
         const QPointF &pos = object->position();
@@ -271,6 +265,17 @@ void OrthogonalRenderer::drawTileLayer(QPainter *painter,
                                        const TileLayer *layer,
                                        const QRectF &exposed) const
 {
+    CellRenderer renderer(painter, this, layer->effectiveTintColor());
+    auto tileRenderFunction = [&renderer](const Cell &cell, const QPointF &pos, const QSizeF &size) {
+        renderer.render(cell, pos, size, CellRenderer::BottomLeft);
+    };
+    drawTileLayer(layer, tileRenderFunction, exposed);
+}
+
+void OrthogonalRenderer::drawTileLayer(const TileLayer *layer,
+                                       const RenderTileCallback &renderTile,
+                                       const QRectF &exposed) const
+{
 
     const int tileWidth = map()->tileWidth();
     const int tileHeight = map()->tileHeight();
@@ -308,11 +313,6 @@ void OrthogonalRenderer::drawTileLayer(QPainter *painter,
     if (startX > endX || startY > endY)
         return;
 
-    const QTransform savedTransform = painter->transform();
-    painter->translate(layerPos);
-
-    CellRenderer renderer(painter, this);
-
     Map::RenderOrder renderOrder = map()->renderOrder();
 
     int incX = 1, incY = 1;
@@ -344,18 +344,11 @@ void OrthogonalRenderer::drawTileLayer(QPainter *painter,
             if (cell.isEmpty())
                 continue;
 
-            Tile *tile = cell.tile();
-            QSize size = (tile && !tile->image().isNull()) ? tile->size() : map()->tileSize();
-            renderer.render(cell,
-                            QPointF(x * tileWidth, (y + 1) * tileHeight),
-                            size,
-                            CellRenderer::BottomLeft);
+            const Tile *tile = cell.tile();
+            const QSize size = (tile && !tile->image().isNull()) ? tile->size() : map()->tileSize();
+            renderTile(cell, layerPos + QPointF(x * tileWidth, (y + 1) * tileHeight), size);
         }
     }
-
-    renderer.flush();
-
-    painter->setTransform(savedTransform);
 }
 
 void OrthogonalRenderer::drawTileSelection(QPainter *painter,
@@ -363,16 +356,29 @@ void OrthogonalRenderer::drawTileSelection(QPainter *painter,
                                            const QColor &color,
                                            const QRectF &exposed) const
 {
+    QPainterPath path;
+
 #if QT_VERSION < 0x050800
     const auto rects = region.rects();
     for (const QRect &r : rects) {
 #else
     for (const QRect &r : region) {
 #endif
-        const QRectF toFill = QRectF(boundingRect(r)).intersected(exposed);
-        if (!toFill.isEmpty())
-            painter->fillRect(toFill, color);
+        const QRectF toFill = QRectF(boundingRect(r));
+        if (toFill.intersects(exposed))
+            path.addRect(toFill);
     }
+
+    QColor penColor(color);
+    penColor.setAlpha(255);
+
+    QPen pen(penColor);
+    pen.setCosmetic(true);
+
+    painter->setPen(pen);
+    painter->setBrush(color);
+    painter->setRenderHint(QPainter::Antialiasing, false);
+    painter->drawPath(path.simplified());
 }
 
 void OrthogonalRenderer::drawMapObject(QPainter *painter,
@@ -381,39 +387,43 @@ void OrthogonalRenderer::drawMapObject(QPainter *painter,
 {
     painter->save();
 
-    const QRectF bounds = object->bounds();
-    QRectF rect(bounds);
+    QRectF bounds = object->bounds();
+    bounds.translate(-alignmentOffset(bounds, object->alignment(map())));
 
-    painter->translate(rect.topLeft());
-    rect.moveTopLeft(QPointF(0, 0));
+    painter->translate(bounds.topLeft());
+    bounds.moveTopLeft(QPointF(0, 0));
 
     const Cell &cell = object->cell();
 
     if (!cell.isEmpty()) {
-        const QSizeF size = object->size();
-        CellRenderer(painter, this).render(cell, QPointF(), size,
-                                           CellRenderer::BottomLeft);
+        CellRenderer(painter, this, object->objectGroup()->effectiveTintColor())
+                .render(cell, QPointF(), bounds.size());
 
         if (testFlag(ShowTileObjectOutlines)) {
-            QPointF tileOffset;
-
-            if (const Tile *tile = cell.tile())
-                tileOffset = tile->offset();
-
-            QRectF rect(QPointF(tileOffset.x(),
-                                tileOffset.y() - size.height()),
-                        size);
+            if (const Tile *tile = object->cell().tile()) {
+                QPointF tileOffset = tile->offset();
+                const QSize tileSize = tile->size();
+                if (!tileSize.isNull()) {
+                    const QSizeF scale {
+                        bounds.width() / tileSize.width(),
+                        bounds.height() / tileSize.height()
+                    };
+                    tileOffset.rx() *= scale.width();
+                    tileOffset.ry() *= scale.height();
+                }
+                bounds.translate(tileOffset);
+            }
 
             QPen pen(Qt::SolidLine);
             pen.setCosmetic(true);
             painter->setRenderHint(QPainter::Antialiasing, false);
             painter->setBrush(Qt::NoBrush);
             painter->setPen(pen);
-            painter->drawRect(rect);
+            painter->drawRect(bounds);
             pen.setStyle(Qt::DotLine);
             pen.setColor(color);
             painter->setPen(pen);
-            painter->drawRect(rect);
+            painter->drawRect(bounds);
         }
     } else {
         const qreal lineWidth = objectLineWidth();
@@ -438,22 +448,22 @@ void OrthogonalRenderer::drawMapObject(QPainter *painter,
         // QCoreGraphicsPaintEngine. Draw them as rectangle instead.
         MapObject::Shape shape = object->shape();
         if (shape == MapObject::Ellipse &&
-                ((rect.width() == qreal(0)) ^ (rect.height() == qreal(0)))) {
+                ((bounds.width() == qreal(0)) ^ (bounds.height() == qreal(0)))) {
             shape = MapObject::Rectangle;
         }
 
         switch (shape) {
         case MapObject::Rectangle: {
-            if (rect.isNull())
-                rect = QRectF(QPointF(-10, -10), QSizeF(20, 20));
+            if (bounds.isNull())
+                bounds = QRectF(QPointF(-10, -10), QSizeF(20, 20));
 
             // Draw the shadow
             painter->setPen(shadowPen);
-            painter->drawRect(rect.translated(shadowOffset));
+            painter->drawRect(bounds.translated(shadowOffset));
 
             painter->setPen(linePen);
             painter->setBrush(fillBrush);
-            painter->drawRect(rect);
+            painter->drawRect(bounds);
             break;
         }
 
@@ -500,16 +510,16 @@ void OrthogonalRenderer::drawMapObject(QPainter *painter,
         }
 
         case MapObject::Ellipse: {
-            if (rect.isNull())
-                rect = QRectF(QPointF(-10, -10), QSizeF(20, 20));
+            if (bounds.isNull())
+                bounds = QRectF(QPointF(-10, -10), QSizeF(20, 20));
 
             // Draw the shadow
             painter->setPen(shadowPen);
-            painter->drawEllipse(rect.translated(shadowOffset));
+            painter->drawEllipse(bounds.translated(shadowOffset));
 
             painter->setPen(linePen);
             painter->setBrush(fillBrush);
-            painter->drawEllipse(rect);
+            painter->drawEllipse(bounds);
             break;
         }
 
@@ -517,7 +527,7 @@ void OrthogonalRenderer::drawMapObject(QPainter *painter,
             const auto& textData = object->textData();
             painter->setFont(textData.font);
             painter->setPen(textData.color);
-            painter->drawText(rect, textData.text, textData.textOption());
+            painter->drawText(bounds, textData.text, textData.textOption());
             break;
         }
         case MapObject::Point: {
