@@ -59,6 +59,7 @@ Tileset::Tileset(QString name, int tileWidth, int tileHeight,
     mTileHeight(tileHeight),
     mTileSpacing(tileSpacing),
     mMargin(margin),
+    mObjectAlignment(Unspecified),
     mOrientation(Orthogonal),
     mGridSize(tileWidth, tileHeight),
     mColumnCount(0),
@@ -261,8 +262,7 @@ bool Tileset::loadFromImage(const QImage &image, const QUrl &source)
  */
 bool Tileset::loadFromImage(const QImage &image, const QString &source)
 {
-    const QUrl url(source);
-    return loadFromImage(image, url.isRelative() ? QUrl::fromLocalFile(source) : url);
+    return loadFromImage(image, Tiled::toUrl(source));
 }
 
 /**
@@ -404,15 +404,14 @@ void Tileset::setImageSource(const QUrl &imageSource)
 
 /**
  * Exists only because the Python plugin interface does not handle QUrl (would
- * be nice to add this). Assumes \a source is a local file when it would
- * otherwise be a relative URL (without scheme).
+ * be nice to add this). Assumes \a source is a local file when it is either
+ * an absolute file path or would otherwise be a relative URL (without scheme).
  *
  * \sa loadFromImage
  */
 void Tileset::setImageSource(const QString &source)
 {
-    const QUrl url(source);
-    setImageSource(url.isRelative() ? QUrl::fromLocalFile(source) : url);
+    setImageSource(Tiled::toUrl(source));
 }
 
 /**
@@ -668,7 +667,7 @@ void Tileset::addWangSet(WangSet *wangSet)
     mWangSets.append(wangSet);
 }
 
-void Tileset::addWangSet(std::unique_ptr<WangSet> &&wangSet)
+void Tileset::addWangSet(std::unique_ptr<WangSet> wangSet)
 {
     addWangSet(wangSet.release());
 }
@@ -856,6 +855,7 @@ SharedTileset Tileset::clone() const
 
     // mFileName stays empty
     c->mTileOffset = mTileOffset;
+    c->mObjectAlignment = mObjectAlignment;
     c->mOrientation = mOrientation;
     c->mGridSize = mGridSize;
     c->mColumnCount = mColumnCount;
