@@ -79,7 +79,7 @@ public:
                             QRect bounds,
                             int compressionLevel);
     void writeObjectGroup(const Tiled::ObjectGroup *,
-                          const QByteArray &key = QByteArray());
+                          const char *key = nullptr);
     void writeImageLayer(const Tiled::ImageLayer *);
     void writeGroupLayer(const Tiled::GroupLayer *,
                          Tiled::Map::LayerDataFormat,
@@ -87,6 +87,7 @@ public:
                          QSize chunkSize);
     void writeMapObject(const Tiled::MapObject *);
 
+    void writeLayerProperties(const Tiled::Layer *);
     void writePolygon(const Tiled::MapObject *);
     void writeTextProperties(const Tiled::MapObject *);
     void writeColor(const char *name, const QColor &color);
@@ -205,7 +206,7 @@ void LuaWriter::writeMap(const Map *map)
     mWriter.writeStartDocument();
     mWriter.writeStartReturnTable();
 
-    mWriter.writeKeyAndValue("version", "1.2");
+    mWriter.writeKeyAndValue("version", "1.4");
     mWriter.writeKeyAndValue("luaversion", "5.1");
     mWriter.writeKeyAndValue("tiledversion", QCoreApplication::applicationVersion());
 
@@ -261,8 +262,16 @@ void LuaWriter::writeProperties(const Properties &properties)
     Properties::const_iterator it = properties.constBegin();
     Properties::const_iterator it_end = properties.constEnd();
     for (; it != it_end; ++it) {
-        const QVariant value = toExportValue(it.value(), mDir);
-        mWriter.writeQuotedKeyAndValue(it.key(), value);
+        if (it.value().userType() == objectRefTypeId()) {
+            mWriter.writeStartTable(it.key());
+            mWriter.setSuppressNewlines(true);
+            mWriter.writeKeyAndValue("id", it.value().value<ObjectRef>().id);
+            mWriter.writeEndTable();
+            mWriter.setSuppressNewlines(false);
+        } else {
+            const QVariant value = toExportValue(it.value(), mDir);
+            mWriter.writeQuotedKeyAndValue(it.key(), value);
+        }
     }
 
     mWriter.writeEndTable();
@@ -298,7 +307,7 @@ void LuaWriter::writeTileset(const Tileset &tileset,
         mWriter.writeStartReturnTable();
 
         // Include version in external tilesets
-        mWriter.writeKeyAndValue("version", "1.2");
+        mWriter.writeKeyAndValue("version", "1.4");
         mWriter.writeKeyAndValue("luaversion", "5.1");
         mWriter.writeKeyAndValue("tiledversion", QCoreApplication::applicationVersion());
     }
@@ -330,13 +339,15 @@ void LuaWriter::writeTileset(const Tileset &tileset,
 
     if (tileset.transparentColor().isValid()) {
         mWriter.writeKeyAndValue("transparentcolor",
-                                tileset.transparentColor().name());
+                                 tileset.transparentColor().name());
     }
 
     const QColor &backgroundColor = tileset.backgroundColor();
     if (backgroundColor.isValid())
         writeColor("backgroundcolor", backgroundColor);
 
+    mWriter.writeKeyAndValue("objectalignment", alignmentToString(tileset.objectAlignment()));
+
     const QPoint offset = tileset.tileOffset();
     mWriter.writeStartTable("tileoffset");
     mWriter.writeKeyAndValue("x", offset.x());
@@ -464,19 +475,12 @@ void LuaWriter::writeTileLayer(const TileLayer *tileLayer,
     mWriter.writeStartTable();
 
     mWriter.writeKeyAndValue("type", "tilelayer");
-    mWriter.writeKeyAndValue("id", tileLayer->id());
-    mWriter.writeKeyAndValue("name", tileLayer->name());
     mWriter.writeKeyAndValue("x", tileLayer->x());
     mWriter.writeKeyAndValue("y", tileLayer->y());
     mWriter.writeKeyAndValue("width", tileLayer->width());
     mWriter.writeKeyAndValue("height", tileLayer->height());
-    mWriter.writeKeyAndValue("visible", tileLayer->isVisible());
-    mWriter.writeKeyAndValue("opacity", tileLayer->opacity());
-
-    const QPointF offset = tileLayer->offset();
-    mWriter.writeKeyAndValue("offsetx", offset.x());
-    mWriter.writeKeyAndValue("offsety", offset.y());
 
+    writeLayerProperties(tileLayer);
     writeProperties(tileLayer->properties());
 
     switch (format) {
@@ -562,26 +566,17 @@ void LuaWriter::writeTileLayerData(const TileLayer *tileLayer,
 }
 
 void LuaWriter::writeObjectGroup(const ObjectGroup *objectGroup,
-                                 const QByteArray &key)
+                                 const char *key)
 {
-    if (key.isEmpty())
-        mWriter.writeStartTable();
-    else
+    if (key)
         mWriter.writeStartTable(key);
+    else
+        mWriter.writeStartTable();
 
     mWriter.writeKeyAndValue("type", "objectgroup");
-    if (objectGroup->id() != 0)
-        mWriter.writeKeyAndValue("id", objectGroup->id());
-    mWriter.writeKeyAndValue("name", objectGroup->name());
-    mWriter.writeKeyAndValue("visible", objectGroup->isVisible());
-    mWriter.writeKeyAndValue("opacity", objectGroup->opacity());
-
-    const QPointF offset = objectGroup->offset();
-    mWriter.writeKeyAndValue("offsetx", offset.x());
-    mWriter.writeKeyAndValue("offsety", offset.y());
-
     mWriter.writeKeyAndValue("draworder", drawOrderToString(objectGroup->drawOrder()));
 
+    writeLayerProperties(objectGroup);
     writeProperties(objectGroup->properties());
 
     mWriter.writeStartTable("objects");
@@ -597,23 +592,16 @@ void LuaWriter::writeImageLayer(const ImageLayer *imageLayer)
     mWriter.writeStartTable();
 
     mWriter.writeKeyAndValue("type", "imagelayer");
-    mWriter.writeKeyAndValue("id", imageLayer->id());
-    mWriter.writeKeyAndValue("name", imageLayer->name());
-    mWriter.writeKeyAndValue("visible", imageLayer->isVisible());
-    mWriter.writeKeyAndValue("opacity", imageLayer->opacity());
-
-    const QPointF offset = imageLayer->offset();
-    mWriter.writeKeyAndValue("offsetx", offset.x());
-    mWriter.writeKeyAndValue("offsety", offset.y());
 
     const QString rel = toFileReference(imageLayer->imageSource(), mDir);
     mWriter.writeKeyAndValue("image", rel);
 
     if (imageLayer->transparentColor().isValid()) {
         mWriter.writeKeyAndValue("transparentcolor",
-                                imageLayer->transparentColor().name());
+                                 imageLayer->transparentColor().name());
     }
 
+    writeLayerProperties(imageLayer);
     writeProperties(imageLayer->properties());
 
     mWriter.writeEndTable();
@@ -627,15 +615,8 @@ void LuaWriter::writeGroupLayer(const GroupLayer *groupLayer,
     mWriter.writeStartTable();
 
     mWriter.writeKeyAndValue("type", "group");
-    mWriter.writeKeyAndValue("id", groupLayer->id());
-    mWriter.writeKeyAndValue("name", groupLayer->name());
-    mWriter.writeKeyAndValue("visible", groupLayer->isVisible());
-    mWriter.writeKeyAndValue("opacity", groupLayer->opacity());
-
-    const QPointF offset = groupLayer->offset();
-    mWriter.writeKeyAndValue("offsetx", offset.x());
-    mWriter.writeKeyAndValue("offsety", offset.y());
 
+    writeLayerProperties(groupLayer);
     writeProperties(groupLayer->properties());
 
     writeLayers(groupLayer->layers(), format, compressionLevel, chunkSize);
@@ -707,6 +688,22 @@ void LuaWriter::writeMapObject(const Tiled::MapObject *mapObject)
     mWriter.writeEndTable();
 }
 
+void LuaWriter::writeLayerProperties(const Layer *layer)
+{
+    if (layer->id() != 0)
+        mWriter.writeKeyAndValue("id", layer->id());
+    mWriter.writeKeyAndValue("name", layer->name());
+    mWriter.writeKeyAndValue("visible", layer->isVisible());
+    mWriter.writeKeyAndValue("opacity", layer->opacity());
+
+    const QPointF offset = layer->offset();
+    mWriter.writeKeyAndValue("offsetx", offset.x());
+    mWriter.writeKeyAndValue("offsety", offset.y());
+
+    if (layer->tintColor().isValid())
+        writeColor("tintcolor", layer->tintColor());
+}
+
 void LuaWriter::writePolygon(const MapObject *mapObject)
 {
     if (mapObject->shape() == MapObject::Polygon)
