@@ -32,11 +32,16 @@
 #include "logginginterface.h"
 #include "mapeditor.h"
 #include "mapview.h"
+#include "preferences.h"
+#include "project.h"
+#include "projectmanager.h"
 #include "regionvaluetype.h"
 #include "scriptedaction.h"
 #include "scriptedfileformat.h"
 #include "scriptedtool.h"
 #include "scriptfile.h"
+#include "scriptfileformatwrappers.h"
+#include "scriptfileinfo.h"
 #include "scriptmodule.h"
 #include "tilecollisiondock.h"
 #include "tilelayer.h"
@@ -50,25 +55,27 @@
 #include <QStandardPaths>
 #include <QTextCodec>
 #include <QtDebug>
+#include <QCoreApplication>
 
 namespace Tiled {
 
-std::unique_ptr<ScriptManager> ScriptManager::mInstance;
+ScriptManager *ScriptManager::mInstance;
 
 ScriptManager &ScriptManager::instance()
 {
     if (!mInstance)
-        mInstance.reset(new ScriptManager);
+        mInstance = new ScriptManager;
     return *mInstance;
 }
 
 void ScriptManager::deleteInstance()
 {
-    mInstance.reset();
+    delete mInstance;
+    mInstance = nullptr;
 }
 
 /*
- * mJSEngine needs to be QQmlEngine for the "Qt" module to be available, which
+ * mEngine needs to be QQmlEngine for the "Qt" module to be available, which
  * is necessary to pass things like QSize or QPoint to some API functions
  * (using Qt.size and Qt.point).
  *
@@ -78,8 +85,6 @@ void ScriptManager::deleteInstance()
 
 ScriptManager::ScriptManager(QObject *parent)
     : QObject(parent)
-    , mEngine(new QQmlEngine(this))
-    , mModule(new ScriptModule(this))
 {
     qRegisterMetaType<Cell>();
     qRegisterMetaType<EditableAsset*>();
@@ -104,46 +109,45 @@ ScriptManager::ScriptManager(QObject *parent)
     qRegisterMetaType<TileLayerEdit*>();
     qRegisterMetaType<TilesetDock*>();
     qRegisterMetaType<TilesetEditor*>();
+    qRegisterMetaType<ScriptMapFormatWrapper*>();
+    qRegisterMetaType<ScriptTilesetFormatWrapper*>();
 
     connect(&mWatcher, &FileSystemWatcher::pathsChanged,
             this, &ScriptManager::scriptFilesChanged);
 
-    const QString configLocation { QStandardPaths::writableLocation(QStandardPaths::AppConfigLocation) };
+    connect(ProjectManager::instance(), &ProjectManager::projectChanged,
+            this, &ScriptManager::refreshExtensionsPaths);
+
+    const QString configLocation { Preferences::configLocation() };
     if (!configLocation.isEmpty()) {
         mExtensionsPath = QDir{configLocation}.filePath(QStringLiteral("extensions"));
 
         if (!QFile::exists(mExtensionsPath))
             QDir().mkpath(mExtensionsPath);
-
-        mExtensionsPaths.append(mExtensionsPath);
     }
 }
 
-void ScriptManager::initialize()
+void ScriptManager::ensureInitialized()
 {
-    QJSValue globalObject = mEngine->globalObject();
-    globalObject.setProperty(QStringLiteral("tiled"), mEngine->newQObject(mModule));
-#if QT_VERSION >= 0x050800
-    globalObject.setProperty(QStringLiteral("TextFile"), mEngine->newQMetaObject<ScriptTextFile>());
-    globalObject.setProperty(QStringLiteral("BinaryFile"), mEngine->newQMetaObject<ScriptBinaryFile>());
-    globalObject.setProperty(QStringLiteral("Layer"), mEngine->newQMetaObject<EditableLayer>());
-    globalObject.setProperty(QStringLiteral("MapObject"), mEngine->newQMetaObject<EditableMapObject>());
-    globalObject.setProperty(QStringLiteral("ObjectGroup"), mEngine->newQMetaObject<EditableObjectGroup>());
-    globalObject.setProperty(QStringLiteral("Terrain"), mEngine->newQMetaObject<EditableTerrain>());
-    globalObject.setProperty(QStringLiteral("Tile"), mEngine->newQMetaObject<EditableTile>());
-    globalObject.setProperty(QStringLiteral("TileLayer"), mEngine->newQMetaObject<EditableTileLayer>());
-    globalObject.setProperty(QStringLiteral("TileMap"), mEngine->newQMetaObject<EditableMap>());
-    globalObject.setProperty(QStringLiteral("Tileset"), mEngine->newQMetaObject<EditableTileset>());
-#endif
+    if (!mEngine) {
+        if (mExtensionsPaths.isEmpty())
+            refreshExtensionsPaths();
 
-    loadExtensions();
+        initialize();
+    }
 }
 
 QJSValue ScriptManager::evaluate(const QString &program,
                                  const QString &fileName, int lineNumber)
 {
+    QJSValue globalObject = mEngine->globalObject();
+    if (!fileName.isEmpty())
+        globalObject.setProperty(QStringLiteral("__filename"), fileName);
+
     QJSValue result = mEngine->evaluate(program, fileName, lineNumber);
     checkError(result, program);
+
+    globalObject.deleteProperty(QStringLiteral("__filename"));
     return result;
 }
 
@@ -173,6 +177,13 @@ QJSValue ScriptManager::evaluateFile(const QString &fileName)
     return evaluate(script, fileName);
 }
 
+QString ScriptManager::createTempValue(const QJSValue &value)
+{
+    auto name = QLatin1Char('$') + QString::number(mTempCount++);
+    mEngine->globalObject().setProperty(name, value);
+    return name;
+}
+
 void ScriptManager::loadExtensions()
 {
     QStringList extensionSearchPaths;
@@ -251,18 +262,51 @@ void ScriptManager::throwError(const QString &message)
 #endif
 }
 
+void ScriptManager::throwNullArgError(int argNumber)
+{
+    throwError(QCoreApplication::translate("Script Errors",
+                                           "Argument %1 is undefined or the wrong type").arg(argNumber));
+}
+
 void ScriptManager::reset()
 {
     Tiled::INFO(tr("Resetting script engine"));
 
     mWatcher.clear();
+
     delete mEngine;
     delete mModule;
 
+    mEngine = nullptr;
+    mModule = nullptr;
+    mTempCount = 0;
+
+    initialize();
+}
+
+void ScriptManager::initialize()
+{
     mEngine = new QQmlEngine(this);
     mModule = new ScriptModule(this);
 
-    initialize();
+    QJSValue globalObject = mEngine->globalObject();
+    globalObject.setProperty(QStringLiteral("tiled"), mEngine->newQObject(mModule));
+#if QT_VERSION >= 0x050800
+    globalObject.setProperty(QStringLiteral("TextFile"), mEngine->newQMetaObject<ScriptTextFile>());
+    globalObject.setProperty(QStringLiteral("BinaryFile"), mEngine->newQMetaObject<ScriptBinaryFile>());
+    globalObject.setProperty(QStringLiteral("Layer"), mEngine->newQMetaObject<EditableLayer>());
+    globalObject.setProperty(QStringLiteral("MapObject"), mEngine->newQMetaObject<EditableMapObject>());
+    globalObject.setProperty(QStringLiteral("ObjectGroup"), mEngine->newQMetaObject<EditableObjectGroup>());
+    globalObject.setProperty(QStringLiteral("Terrain"), mEngine->newQMetaObject<EditableTerrain>());
+    globalObject.setProperty(QStringLiteral("Tile"), mEngine->newQMetaObject<EditableTile>());
+    globalObject.setProperty(QStringLiteral("TileLayer"), mEngine->newQMetaObject<EditableTileLayer>());
+    globalObject.setProperty(QStringLiteral("TileMap"), mEngine->newQMetaObject<EditableMap>());
+    globalObject.setProperty(QStringLiteral("Tileset"), mEngine->newQMetaObject<EditableTileset>());
+#endif
+
+    registerFileInfo(mEngine);
+
+    loadExtensions();
 }
 
 void ScriptManager::scriptFilesChanged(const QStringList &scriptFiles)
@@ -271,4 +315,33 @@ void ScriptManager::scriptFilesChanged(const QStringList &scriptFiles)
     reset();
 }
 
+void ScriptManager::refreshExtensionsPaths()
+{
+    QStringList extensionsPaths;
+
+    if (!mExtensionsPath.isEmpty())
+        extensionsPaths.append(mExtensionsPath);
+
+    // Add extensions path from project
+    auto &projectExtensionsPath = ProjectManager::instance()->project().mExtensionsPath;
+    if (!projectExtensionsPath.isEmpty()) {
+        const QFileInfo info(projectExtensionsPath);
+        if (info.exists() && info.isDir())
+            extensionsPaths.append(projectExtensionsPath);
+    }
+
+    extensionsPaths.sort();
+    extensionsPaths.removeDuplicates();
+
+    if (extensionsPaths == mExtensionsPaths)
+        return;
+
+    mExtensionsPaths.swap(extensionsPaths);
+
+    if (mEngine) {
+        Tiled::INFO(tr("Extensions paths changed: %1").arg(mExtensionsPaths.join(QLatin1String(", "))));
+        reset();
+    }
+}
+
 } // namespace Tiled
